==> mojo2-runtime-api-0.13.5-sources.jar: ai/h2o/mojos/runtime/utils/DateParser.java <==
package ai.h2o.mojos.runtime.utils;

import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.DateTimeFormatterBuilder;
import org.joda.time.format.DateTimeParser;

import java.util.ArrayList;
import java.util.List;

import static ai.h2o.mojos.runtime.utils.Consts.getSysProp;
import static ai.h2o.mojos.runtime.utils.ArrayUtils.join;

public class DateParser {

  static {
    DateTimeZone.setDefault(DateTimeZone.UTC);
  }

  /* User can define their own formats */
  static String[] USER_DEFINED_FORMATS = Consts.getSysProp("date.formats", "").split(",");

  public final static String[] SUPPORTED_FORMATS = {
      "M[M]/d[d]/yyyy[' ']['T'][' ']h[h]:mm:ss[' ']a",
      "M[M]/d[d]/yyyy[' ']['T'][' ']h[h]:mm:ss.SSS[' ']a",
      "M[M]/d[d]/yyyy[' ']['T'][' ']H[H]:mm:ss",
      "M[M]/d[d]/yyyy[' ']['T'][' ']H[H]:mm:ss.SSS",
      "M[M]/d[d]/yyyy",
      "M[M]-d[d]-yyyy[' ']['T'][' ']h[h]:mm:ss[' ']a",
      "M[M]-d[d]-yyyy[' ']['T'][' ']h[h]:mm:ss.SSS[' ']a",
      "M[M]-d[d]-yyyy[' ']['T'][' ']H[H]:mm:ss",
      "M[M]-d[d]-yyyy[' ']['T'][' ']H[H]:mm:ss.SSS",
      "M[M]-d[d]-yyyy",
      "d[d].M[M].yyyy[' ']['T'][' ']h[h]:mm:ss[' ']a",
      "d[d].M[M].yyyy[' ']['T'][' ']h[h]:mm:ss.SSS[' ']a",
      "d[d].M[M].yyyy[' ']['T'][' ']H[H]:mm:ss",
      "d[d].M[M].yyyy[' ']['T'][' ']H[H]:mm:ss.SSS",
      "d[d].M[M].yyyy",
      "yyyy/M[M]/d[d]' 'HH:mm:ss",
      "yyyy/M[M]/d[d]' 'HH:mm:ss.SSS",
      "yyyy/M[M]/d[d]",
      "yyyy-M[M]-d[d]' 'HH:mm:ss",
      "yyyy-M[M]-d[d]' 'HH:mm:ss.SSS",
      "yyyy-M[M]-d[d]'T'HH:mm:ss.SSSSSSSSS", // Datatable format
      "yyyy-M[M]-d[d]",
      "h[h]:mm:ss[' ']a",
      "h[h]:mm:ss.SSS[' ']a",
      "H[H]:mm:ss",
      "H[H]:mm:ss.SSS",
      "h[h]:mm[' ']a",
      "H[H]:mm",
  };
  
  public static DateTimeFormatter DTF = new DateTimeFormatterBuilder()
      .append(null,
              toFormatter(ArrayUtils.join(
                  SUPPORTED_FORMATS,
                  USER_DEFINED_FORMATS
              ))
      ).toFormatter();

  static DateTimeParser[] toFormatter(String[] formats) {
    List<String> expandedFormats = expandFormats(formats);
    DateTimeParser[] result = new DateTimeParser[expandedFormats.size()];
    int i = 0;
    for (String f : expandedFormats) {
      result[i++] = DateTimeFormat.forPattern(f).getParser();
    }
    return result;
  }

  static List<String> expandFormats(String[] formats) {
    ArrayList<String> expandedFormats = new ArrayList<>();
    for (String format : formats) {
      expandFormat(new StringBuilder(format), 0, expandedFormats);
    }
    return expandedFormats;
  }

  static List<String> expandFormat(StringBuilder format, int startIdx, List<String> result) {
    StringBuilder prefix = new StringBuilder();
    String suffix = null;
    int openGroupIdx = -1;
    int closeGroupIdx = -1;
    int openingCnt = 0;
    for (int i = startIdx; i < format.length(); i++) {
      char c = format.charAt(i);
      switch (c) {
        case '[':
          if (openGroupIdx < 0) {
            openGroupIdx = i;
            prefix.append(format, 0, i);
          }
          openingCnt++;
          break;
        case ']': {
          if (openGroupIdx >= 0) openingCnt--;
          if (openingCnt == 0) {
            closeGroupIdx = i;
            suffix = format.substring(i+1);
          }
        }
        break;
        default:
      }
      if (closeGroupIdx > 0) break;
    }
    if (openGroupIdx >= 0) {
      // Generate formats with optional groups
      String[] groups = format.substring(openGroupIdx+1, closeGroupIdx).split(",");
      for (String group : groups) {
        expandFormat(new StringBuilder(prefix).append(group).append(suffix), openGroupIdx, result);
      }
      // Generate formats without the optional group
      expandFormat(format.delete(openGroupIdx, closeGroupIdx + 1), closeGroupIdx - (closeGroupIdx - openGroupIdx), result);
      return result;
    } else {
      if (format.length() > 0)
        result.add(format.toString());
      return result;
    }
  }
}

==> mojo2-runtime-api-0.13.7-sources.jar: ai/h2o/mojos/runtime/utils/DateParser.java <==
package ai.h2o.mojos.runtime.utils;

import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.DateTimeFormatterBuilder;
import org.joda.time.format.DateTimeParser;

import java.util.ArrayList;
import java.util.List;

import static ai.h2o.mojos.runtime.utils.Consts.getSysProp;
import static ai.h2o.mojos.runtime.utils.ArrayUtils.join;

public class DateParser {

  static {
    DateTimeZone.setDefault(DateTimeZone.UTC);
  }

  /* User can define their own formats */
  static String[] USER_DEFINED_FORMATS = Consts.getSysProp("date.formats", "").split(",");

  public final static String[] SUPPORTED_FORMATS = {
      "M[M]/d[d]/yyyy[' ']['T'][' ']h[h]:mm:ss[' ']a",
      "M[M]/d[d]/yyyy[' ']['T'][' ']h[h]:mm:ss.SSS[' ']a",
      "M[M]/d[d]/yyyy[' ']['T'][' ']H[H]:mm:ss",
      "M[M]/d[d]/yyyy[' ']['T'][' ']H[H]:mm:ss.SSS",
      "M[M]/d[d]/yyyy",
      "M[M]-d[d]-yyyy[' ']['T'][' ']h[h]:mm:ss[' ']a",
      "M[M]-d[d]-yyyy[' ']['T'][' ']h[h]:mm:ss.SSS[' ']a",
      "M[M]-d[d]-yyyy[' ']['T'][' ']H[H]:mm:ss",
      "M[M]-d[d]-yyyy[' ']['T'][' ']H[H]:mm:ss.SSS",
      "M[M]-d[d]-yyyy",
      "d[d].M[M].yyyy[' ']['T'][' ']h[h]:mm:ss[' ']a",
      "d[d].M[M].yyyy[' ']['T'][' ']h[h]:mm:ss.SSS[' ']a",
      "d[d].M[M].yyyy[' ']['T'][' ']H[H]:mm:ss",
      "d[d].M[M].yyyy[' ']['T'][' ']H[H]:mm:ss.SSS",
      "d[d].M[M].yyyy",
      "yyyy/M[M]/d[d]' 'HH:mm:ss",
      "yyyy/M[M]/d[d]' 'HH:mm:ss.SSS",
      "yyyy/M[M]/d[d]",
      "yyyy-M[M]-d[d]' 'HH:mm:ss",
      "yyyy-M[M]-d[d]' 'HH:mm:ss.SSS",
      "yyyy-M[M]-d[d]'T'HH:mm:ss.SSSSSSSSS", // Datatable format
      "yyyy-M[M]-d[d]",
      "yyyyMMdd", // 8-digit integer date format as requested by Arno. Should be removed when date format strings are used
      "h[h]:mm:ss[' ']a",
      "h[h]:mm:ss.SSS[' ']a",
      "H[H]:mm:ss",
      "H[H]:mm:ss.SSS",
      "h[h]:mm[' ']a",
      "H[H]:mm",
  };
  
  public static DateTimeFormatter DTF = new DateTimeFormatterBuilder()
      .append(null,
              toFormatter(ArrayUtils.join(
                  SUPPORTED_FORMATS,
                  USER_DEFINED_FORMATS
              ))
      ).toFormatter();

  static DateTimeParser[] toFormatter(String[] formats) {
    List<String> expandedFormats = expandFormats(formats);
    DateTimeParser[] result = new DateTimeParser[expandedFormats.size()];
    int i = 0;
    for (String f : expandedFormats) {
      result[i++] = DateTimeFormat.forPattern(f).getParser();
    }
    return result;
  }

  static List<String> expandFormats(String[] formats) {
    ArrayList<String> expandedFormats = new ArrayList<>();
    for (String format : formats) {
      expandFormat(new StringBuilder(format), 0, expandedFormats);
    }
    return expandedFormats;
  }

  static List<String> expandFormat(StringBuilder format, int startIdx, List<String> result) {
    StringBuilder prefix = new StringBuilder();
    String suffix = null;
    int openGroupIdx = -1;
    int closeGroupIdx = -1;
    int openingCnt = 0;
    for (int i = startIdx; i < format.length(); i++) {
      char c = format.charAt(i);
      switch (c) {
        case '[':
          if (openGroupIdx < 0) {
            openGroupIdx = i;
            prefix.append(format, 0, i);
          }
          openingCnt++;
          break;
        case ']': {
          if (openGroupIdx >= 0) openingCnt--;
          if (openingCnt == 0) {
            closeGroupIdx = i;
            suffix = format.substring(i+1);
          }
        }
        break;
        default:
      }
      if (closeGroupIdx > 0) break;
    }
    if (openGroupIdx >= 0) {
      // Generate formats with optional groups
      String[] groups = format.substring(openGroupIdx+1, closeGroupIdx).split(",");
      for (String group : groups) {
        expandFormat(new StringBuilder(prefix).append(group).append(suffix), openGroupIdx, result);
      }
      // Generate formats without the optional group
      expandFormat(format.delete(openGroupIdx, closeGroupIdx + 1), closeGroupIdx - (closeGroupIdx - openGroupIdx), result);
      return result;
    } else {
      if (format.length() > 0)
        result.add(format.toString());
      return result;
    }
  }
}
